/***********************************************
  ARDUINO BATTERY CAPACITY TESTER
  Version-1.0 by deba168,INDIA,04/09/2016
  Version-1.1 by Mat Bohne,USA,02/13/2019
***********************************************/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define OLED_RESET 13
Adafruit_SSD1306 display(OLED_RESET);
#define MOSFET_Pin 2
#define Bat_Pin A1
#define Res_Pin A0
#define Buzzer_Pin 9
#define pb1 4
#define pb2 3
float Capacity = 0.0; // Capacity in mAh
float Res_Value = 5;  // Resistor Value in Ohm
float Vcc = 4.67; // Voltage of Arduino 5V pin ( Mesured by Multimeter )
float Current = 0.0; // Current in Amp
float mA = 0;       // Current in mA
float Bat_Volt = 0.0;  // Battery Voltage
float Res_Volt = 0.0;  // Voltage at lower end of the Resistor
float Bat_High; // Battery High Voltage
float Bat_Low; // Discharge Cut Off Voltage
unsigned long previousMillis = 0; // Previous time in ms
unsigned long millisPassed = 0;  // Current time in ms
float sample1 = 0;
float sample2 = 0;
int x = 0;
int row = 0;
int pb = 0;
int BV;
int TST = 0;
void beep(unsigned char delay_time) {
  analogWrite(9, 20);      // PWM signal to generate beep tone
  delay(delay_time);          // wait for a delayms ms
  analogWrite(Buzzer_Pin, 0);  // 0 turns it off
  delay(delay_time);          // wait for a delayms ms
}

//*******************************Setup Function ***************************************************************

void setup() {
  Serial.begin(9600);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.setTextSize(1);
  display.setTextColor(WHITE);
  pinMode(MOSFET_Pin, OUTPUT);
  pinMode(Buzzer_Pin, OUTPUT);
  pinMode(pb1, INPUT);
  pinMode(pb2, INPUT);
  digitalWrite(MOSFET_Pin, LOW);  // MOSFET is off during the start
  Serial.println(F("CLEARDATA"));
  Serial.println(F("LABEL: Time,Bat_Volt,capacity"));

  Serial.println(F("Arduino Battery Capacity Tester v1.1"));
  selections();
}
//********************************Main Loop Function***********************************************************
void loop() {
  voltage();
  conditions();
  OLEDisplay();
  if (digitalRead(pb2) == LOW ) {
    display.clearDisplay();
    selections();
    delay(300);
  }
}
//********************************Display Information***********************************************************
void OLEDisplay() {
  display.clearDisplay();
  if ( Bat_Volt < BV) {
    display.setCursor(0, 0);
    display.println(F("Cell Not Detected"));
  }
  else if ( Bat_Volt > Bat_High) {
    display.setCursor(0, 0);
    display.println(F("Cell Voltage High"));
  }
  else if (Bat_Volt < Bat_Low) {
    display.setCursor(0, 0);
    display.println(F("Cell Voltage Low"));
  }
  else if (Bat_Volt < Bat_High && Bat_Volt > Bat_Low && TST == 0) {
    display.setCursor(0, 0);
    display.print(F("Cell Voltage Normal "));
  }
  else if (TST == 1) {
    display.setCursor(0, 0);
    display.print(F("Test Mode! "));
  }

  display.setCursor(0, 8);
  display.print(F("Voltage  = "));
  display.print(Bat_Volt, 2);
  display.print(F(" v"));
  display.setCursor(0, 16);
  display.print(F("Currect  = "));
  display.print( mA, 0);
  display.print(F(" mA"));
  display.setCursor(0, 24);
  display.print(F("Capacity = "));
  display.print(Capacity , 1);
  display.print(F(" mAh"));
  display.display();
  Serial.println( mA, 0);

}
//********************************Analyze conditions***********************************************************
void conditions() {
  if ( Bat_Volt > Bat_High) {
    digitalWrite(MOSFET_Pin, LOW); // Turned Off the MOSFET // No discharge
    beep(200);
    Serial.println(F( "Warning High-V! "));
  }
  else if (Bat_Volt < Bat_Low) {
    digitalWrite(MOSFET_Pin, LOW);
    beep(200);
    Serial.println(F( "Warning Low-V! "));
  }
  else if (Bat_Volt > Bat_Low && Bat_Volt < Bat_High  ) { // Check if the battery voltage is within the safe limit
    digitalWrite(MOSFET_Pin, HIGH);
    millisPassed = millis() - previousMillis;
    Current = (Bat_Volt - Res_Volt) / Res_Value;
    mA = Current * 1000.0 ;
    Capacity = Capacity + mA * (millisPassed / 3600000.0); // 1 Hour = 3600000ms
    previousMillis = millis();
    Serial.print(F("DATA,TIME,")); Serial.print(Bat_Volt); Serial.print(","); Serial.println(Capacity);
    row++;
    x++;

  }

}
//********************************Analyze voltage***********************************************************
void voltage() {
  // Vcc = readVcc()/1000.0; // Conevrrt mV to Volt
  // Voltage devider Out = Bat_Volt * R2/(R1+R2 ) // R1 =10K and R2 =10K

  for (int i = 0; i < 100; i++)
  {
    sample1 = sample1 + analogRead(Bat_Pin); //read the voltage from the divider circuit
    delay (2);
  }
  sample1 = sample1 / 100;
  Bat_Volt = 2 * sample1 * Vcc / 1024.0;

  // *********  Measuring Resistor Voltage ***********

  for (int i = 0; i < 100; i++)
  {
    sample2 = sample2 + analogRead(Res_Pin); //read the voltage from the divider circuit
    delay (2);
  }
  sample2 = sample2 / 100;
  Res_Volt = 2 * sample2 * Vcc / 1024.0;

}
//********************************Select Cell chemistry***********************************************************
void selections() {
  delay(500);
  display.clearDisplay();
  display.setCursor(0, 0);
  display.println(F("Select Cell Chemistry"));
  display.display();
  while (1) {
    if (digitalRead(pb1) == LOW && pb == 0) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("LI-ion-(LiCoO2)"));
      display.setCursor(0, 8);
      display.println(F("(LiMn2O4)(LiNiMnCoO2)"));
      display.setCursor(0, 16);
      display.println(F("(LiNiCoAlO2)"));
      display.setCursor(0, 24);
      display.println(F("Low 3.0v, High 4.2v"));
      display.display();
      Serial.println(F("LI-ion-(LiCoO2)(LiMn2O4)(LiNiMnCoO2)(LiNiCoAlO2), Low 3.0v, High 4.2v"));
      TST = 0;
      pb = 1;
      Bat_High = 4.3; // Battery High Voltage
      Bat_Low = 3.0; // Discharge Cut Off Voltage
      BV = .5;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 1) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Li-Iron Phosphate"));
      display.setCursor(0, 8);
      display.println(F("(LiFePO4)"));
      display.setCursor(0, 24);
      display.println(F("Low 2.5v, High 3.7v"));
      display.display();
      Serial.println(F("Li-Iron Phosphate(LiFePO4), low 2.5v, high 3.7v"));
      TST = 0;
      pb = 2;
      Bat_High = 3.75; // Battery High Voltage
      Bat_Low = 2.45; // Discharge Cut Off Voltage
      BV = .5;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 2) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Li-Titanate"));
      display.setCursor(0, 8);
      display.println(F("(Li4Ti5O12)"));
      display.setCursor(0, 24);
      display.println(F("Low 1.8v, High 2.9v"));
      display.display();
      Serial.println(F("Li-Titanate(Li4Ti5O12), low 1.8v, high 2.9v"));
      TST = 0;
      pb = 3;
      Bat_High = 2.95; // Battery High Voltage
      Bat_Low = 1.75; // Discharge Cut Off Voltage
      BV = .5;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 3) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Ni-cadmium(NiCd)"));
      display.setCursor(0, 8);
      display.println(F("Ni-MetalHydride(NiMH)"));
      display.setCursor(0, 16);
      display.println(F("Ni-iron(NiFe)"));
      display.setCursor(0, 24);
      display.println(F("Low 1.0v, High 1.5v"));
      display.display();
      Serial.println(F("Ni-cadmium(NiCd),Ni-MetalHydride(NiMH),Ni-iron(NiFe), low 1.0v, high 1.5v"));
      TST = 0;
      pb = 4;
      Bat_High = 1.6; // Battery High Voltage
      Bat_Low = .95; // Discharge Cut Off Voltage
      BV = .5;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 4) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Lead Acid Cell"));
      display.setCursor(0, 8);
      display.println(F("Pb-H2SO4)"));
      display.setCursor(0, 24);
      display.println(F("Low 1.8v, High 2.4v"));
      display.display();
      Serial.println(F("Lead Acid Cell, Pb-H2SO4), low 1.8v, high 2.4v"));
      TST = 0;
      pb = 5;
      Bat_High = 2.4; // Battery High Voltage
      Bat_Low = 1.7; // Discharge Cut Off Voltage
      BV = .5;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 5) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Galvanic Cell Test:"));
      display.setCursor(0, 24);
      display.println(F("Low 0.2v, High 5.0v"));
      display.display();
      Serial.println(F("Galvanic Cell Test:   low 0.2v, high 5.0v"));
      TST = 0;
      pb = 6;
      Bat_High = 5; // Battery High Voltage
      Bat_Low = .2; // Discharge Cut Off Voltage
      BV = .1;
      delay(300);
    }
    if (digitalRead(pb1) == LOW  && pb == 6) {
      display.clearDisplay();
      display.setCursor(0, 0);
      display.println(F("Test Mode:"));
      display.setCursor(0, 24);
      display.println(F("Low 0.0v, High 5.0v"));
      display.display();
      Serial.println(F("Test Mode:   low 0.0v, high 5.0v"));
      TST = 1;
      pb = 0;
      Bat_High = 5; // Battery High Voltage
      Bat_Low = 0; // Discharge Cut Off Voltage
      BV = 0;
      delay(300);
    }
    if (digitalRead(pb2) == LOW ) {
      display.clearDisplay();
      delay(300);
      break;
    }
  }
}

